﻿using System;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.IO;
using IniParser;
using PCG.Mercurial.IronMercurial;

namespace PCG.Mercurial
{
	public class IronMercurialRepositoryManager : IRepositoryManager
	{
		#region ##############  Instance Fields  ##############
		private IniData _hgrc;
		private IronMercurialService _fehg;
		#endregion

		#region ##############  Constructors  ##############
		public IronMercurialRepositoryManager()
		{
			_fehg = new IronMercurialService(@"C:\Projects\Mercurial\PCG.Mercurial.IronMercurial\Run");
		}


		#endregion


		public static IronMercurialRepositoryManager Create(DirectoryInfo repoLocation)
		{
			return new IronMercurialRepositoryManager { RepositoryLocation = repoLocation };
		}

		#region Implementation of IRepositoryManager

		/// <summary>
		/// The location of the Mercurial repository.
		/// </summary>
		public DirectoryInfo RepositoryLocation { get; set; }

		/// <exception cref="FileNotFoundException"><c>FileNotFoundException</c>.</exception>
		public IniData HgConfiguration
		{
			get
			{
				if (_hgrc == null)
				{
					var rcFilePath = Path.Combine(RepositoryLocation.FullName, ".hg", "hgrc");
					var parser = new FileIniDataParser();
					if (!File.Exists(rcFilePath))
						throw new FileNotFoundException();
					_hgrc = parser.LoadFile(rcFilePath, true);
				}
				return _hgrc;
			}
			set
			{
				var rcFilePath = Path.Combine(RepositoryLocation.FullName, ".hg", "hgrc");
				if (!File.Exists(rcFilePath))
					throw new FileNotFoundException();
				var parser = new FileIniDataParser();
				_hgrc = value;
				parser.SaveFile(rcFilePath, _hgrc);
			}
		}

		/// <summary>
		/// Initializes a new repository in the directory provided.
		/// </summary>
		/// <returns></returns>
		public HgResult Init(DirectoryInfo repoInfo) { throw new NotImplementedException(); }

		public HgResult Clone(string repositoryUrl) { return Clone(repositoryUrl, null, null); }
		public HgResult Clone(string repositoryUrl, string username, string password)
		{
			//Contract.Requires(repositoryUrl != null);

			Trace.WriteLine("Preparing to call `hg clone`", "Repository Manager");

			var command = CloneCommand.CreateCommand(RepositoryLocation, repositoryUrl);

			if (command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if (command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			if (command.Repository.Exists)
				throw new InvalidOperationException("The clone command requires the target directory to be empty.");

			if (!String.IsNullOrEmpty(username) && !String.IsNullOrEmpty(password))
			{
				var newUrl = new UriBuilder(repositoryUrl) {UserName = username, Password = password};
				command.RepoUrl = newUrl.Uri.OriginalString;
			}

			CommandResult result;
			result = _fehg.Execute(command);

			if (result.ReturnType == null && result.ReturnCode == 0)
				return HgResult.CloneSuccess;

			return HgResult.UnspecifiedError;
		}

		public IncomingCommandResult Incoming() { throw new NotImplementedException(); }

		public HgResult Pull() { throw new NotImplementedException(); }

		/// <summary>
		/// Commits a new changeset to the repository.
		/// </summary>
		/// <returns></returns>
		public HgResult CommitAllChanges(DirectoryInfo repoInfo, string commitMessage) { throw new NotImplementedException(); }

		/// <summary>
		/// Adds (a) new file(s) to a repository.
		/// </summary>
		/// <returns></returns>
		public HgResult Add(DirectoryInfo repoInfo, FileInfo[] newFiles) { throw new NotImplementedException(); }

		/// <summary>
		/// Adds all files in a subdirectory to a repository.
		/// </summary>
		/// <returns></returns>
		public HgResult Add(DirectoryInfo repoInfo, DirectoryInfo subDirectory, bool recursively) { throw new NotImplementedException(); }

		#endregion
	}
}