﻿// Copyright:     (C) 2011 by Zachary Gramana
// Author:        Zachary Gramana
// Version:       0.1
// Description:   Invokes hg from inside a C# library.
// StartedOn:     4/29/2011
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using IronPython.Hosting;
using IronPython.Runtime;
using IronPython.Runtime.Exceptions;
using Microsoft.Scripting.Hosting;

namespace PCG.Mercurial.IronMercurial
{
	public class IronMercurialService
	{
		#region ##############  Instance Fields  ##############

		private readonly DirectoryInfo					_runPath;
		private readonly	Dictionary<String, Object>	_ipyOptions;
		private readonly	ScriptEngine				_ipy;
		private readonly	ScriptSource				_hg;
		#endregion

		#region ##############  Constructors  ##############
		public IronMercurialService(String runPath) : this(runPath, null) { }
		public IronMercurialService(String runPath, Dictionary<String,Object> ipyOptions)
		{
			_runPath = new DirectoryInfo(runPath);

			if (!_runPath.Exists)
				throw new InvalidOperationException("The path pointing to IronMercurial's Run folder must exist.");

			_ipyOptions = ipyOptions ?? GetDefaultConfiguration();
			_ipy = Python.CreateEngine(_ipyOptions);

			// Set Python search paths.
			var pythonPaths = GetPythonPaths();
			_ipy.SetSearchPaths(pythonPaths);

			_hg = _ipy.CreateScriptSourceFromFile
				(					
					Path.Combine(_runPath.FullName, @"Scripts\hg.py")
				);
		}
		#endregion

		#region ##############  Methods  ##############

		public int DExecute(IHgCommand command)
		{
			var outputPath = _runPath.FullName;

			var args = new List { Path.Combine(Environment.CurrentDirectory, @"Scripts\hg.py"), "clone", "https://bitbucket.org/zgramana/pcg.rolelayer", outputPath };
			var configOptions = new Dictionary<string, object> { { "FullFrames", true }, { "CompilationThreshold", 1 } };
#if DEBUG
			configOptions["Debug"] = true;	// hits breakpoints in python code as well.				
#else
				configOptions["Debug"] = false;
				configOptions["ShowClrExceptions"] = true; // will print exception to console.
#endif
			args.Add("-v");					// Enables verbose output.
			args.Add("--debug");			// tells hg to log debugging output to console.

			var ipy = Python.CreateEngine(configOptions);
			var hg = ipy.CreateScriptSourceFromFile(args[0].ToString());

			var sys = ipy.GetSysModule();
			sys.SetVariable("argv", args);

			// Set Python search paths.
			var pythonPaths = new List<String>();

			//var paths = Environment.GetEnvironmentVariable("IRONPYTHONPATH")
			var paths = new StringBuilder(Environment.CurrentDirectory + ";");
			paths.Append(Path.Combine(Environment.CurrentDirectory, "Lib") + ";");
			paths.Append(Path.Combine(Environment.CurrentDirectory, "Scripts") + ";");
			paths.Append(Path.Combine(Environment.CurrentDirectory, @"Lib\site-packages") + ";");

			paths.ToString().Split(';').ToList().ForEach(p => { if (!String.IsNullOrWhiteSpace(p)) pythonPaths.Add(p); });

			var iterationsLeft = 2;

			ipy.SetSearchPaths(pythonPaths);

			try
			{
				var returnCode = hg.ExecuteProgram();
				return returnCode;
			}
			catch (Exception ex)
			{
				var message = GetMessage(ex);
				Debug.WriteLine(String.Format("{1}{0}{1}", message, "\r\n\r\n"));
				return -1;
			}
		}

		public CommandResult Execute(IHgCommand command)
		{
			// Create the "commandline args" for the Python script, aka hg.py.
			var commandParts = command.ToString().Split(new[] {' '}, StringSplitOptions.RemoveEmptyEntries);
			var args = new List { Path.Combine(_runPath.FullName,  @"Scripts\hg.py") };
			args.extend(commandParts);

			var sys = _ipy.GetSysModule();
			sys.SetVariable("argv", args);

			Int32 returnCode;
			Object returnType;
			try
			{
				// Run the script as a program.
				returnCode = _hg.ExecuteProgram();
				returnType = null;
			}
			catch (SystemExitException systemExitException)
			{
				returnCode = systemExitException.GetExitCode(out returnType);
			}
			catch (Exception ex)
			{
				var message = GetMessage(ex);

				Debug.WriteLine(String.Format("{1}{0}{1}", message, "\r\n\r\n"));
				returnCode = -1;
				returnType = null;
			}

			return new CommandResult { ReturnCode = returnCode, ReturnType = returnType };
		}

		private string GetMessage(Exception ex) 
		{
			if (_ipy != null)
			{
				var exception = _ipy.GetService<ExceptionOperations>();
				Debug.WriteLine(exception.FormatException(ex));
			}
			return ex.Message;
		}

		private List<string> GetPythonPaths()
		{
			var pythonPaths = new List<String>();

			// When we support using a system-wide IPY + pure HG install,
			// you'll be able to uncomment the two lines below instead.
			//var paths = Environment.GetEnvironmentVariable("IRONPYTHONPATH")
			//paths.ToString().Split(';').ToList().ForEach(p => { if (!String.IsNullOrWhiteSpace(p)) pythonPaths.Add(p); });

			// For now, use the paths we provide.
			pythonPaths.Add(_runPath.FullName);
			pythonPaths.Add(Path.Combine(_runPath.FullName,  "Lib"));
			pythonPaths.Add(Path.Combine(_runPath.FullName, @"Lib\site-packages"));
			pythonPaths.Add(Path.Combine(_runPath.FullName,  "Scripts"));

			return pythonPaths;
		}

		private static Dictionary<String,Object> GetDefaultConfiguration() 
		{
			var configOptions = new Dictionary<string, object> { { "FullFrames", true }, { "CompilationThreshold", 1 } };
			#if DEBUG
			configOptions["Debug"] = true;			   // hits breakpoints in python code as well.				
			#else
			configOptions["ShowClrExceptions"] = true; // will print exception to console.			
			#endif
			return configOptions;
		}

		#endregion

	}
}
