﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.Linq;
using System.IO;
using System.Text;

namespace PCG.Mercurial
{
    public class AddFileCommand : HgCommandBase
    {
        #region Constants
        // ####################################################################
        // ###### Constants                                              ######
        // ####################################################################
        private const String CommandText = "add";

        #endregion

        #region Static Methods
        // ####################################################################
        // ###### Static Methods                                         ######
        // ####################################################################
        public static AddFileCommand CreateCommand(DirectoryInfo repoInfo, IEnumerable<FileInfo> newFiles)
        {
            Contract.Requires(newFiles != null);
            Contract.Requires(repoInfo != null);
            Contract.Ensures(Contract.Result<AddFileCommand>() != null);


            return new AddFileCommand(repoInfo) {Files = newFiles};
        }
        public static AddFileCommand CreateCommand(DirectoryInfo repoInfo, DirectoryInfo searchRoot, SearchOption searchOption, Boolean includeSubrepositories)
        {
            Contract.Requires(searchRoot != null);
            Contract.Requires(repoInfo != null);
            Contract.Ensures(Contract.Result<AddFileCommand>() != null);


            return new AddFileCommand(repoInfo) { SearchDirectory = searchRoot, PatternSearchOption = searchOption, TraverseSubrepositories = includeSubrepositories};
        }
        public static AddFileCommand CreateCommand(DirectoryInfo repoInfo, DirectoryInfo searchRoot, SearchOption searchOption, IEnumerable<String> includeSearchPatterns, IEnumerable<String> excludeSearchPatterns, Boolean includeSubrepositories)
        {
            Contract.Requires(excludeSearchPatterns != null);
            Contract.Requires(includeSearchPatterns != null);
            Contract.Requires(searchRoot != null);
            Contract.Requires(repoInfo != null);
            Contract.Ensures(Contract.Result<AddFileCommand>() != null);


            return new AddFileCommand(repoInfo) { SearchDirectory = searchRoot, IncludePatterns = includeSearchPatterns, ExcludePatterns = excludeSearchPatterns, PatternSearchOption = searchOption, TraverseSubrepositories = includeSubrepositories};
        }


        #endregion

        #region Constructors
        // ####################################################################
        // ###### Constructors                                           ######
        // ####################################################################
        private AddFileCommand(DirectoryInfo repoInfo) : base(repoInfo) { }


        #endregion

        #region Properties
        // ####################################################################
        // ###### Properties                                             ######
        // ####################################################################
        public DirectoryInfo SearchDirectory { get; set; }
        public IEnumerable<String> IncludePatterns { get; set; }
        public IEnumerable<String> ExcludePatterns { get; set; }
        public SearchOption PatternSearchOption { get; set; }
        public Boolean TraverseSubrepositories { get; set; }
        public Boolean DryRunOnly { get; set; }
        public IEnumerable<FileInfo> Files { get; set; }


        #endregion

        #region Overrides of HgCommandBase
        // ####################################################################
        // ###### Overrides of HgCommandBase                             ######
        // ####################################################################
        /// <summary>
        /// Returns a properly populated command result based on the results of the command's execution.
        /// </summary>
        /// <param name="hgProcess"></param>
        /// <returns></returns>
        public override IHgCommandResult GetCommandResult(Process hgProcess)
        {
            Contract.Requires(hgProcess != null);
            
            return hgProcess.HasExited ? new AddFileCommandResult(hgProcess) : null;
        }
        public override string ToString()
        {
            var commandArgs = new StringBuilder(CommandText);

            if (DryRunOnly)
                commandArgs.Append(" -n");

            if (Files != null && Files.Count() > 0)
            {
                foreach (var fileInfo in Files)
                {
                    commandArgs.Append(String.Concat(" ", fileInfo.FullName));
                }
            }
            else if (SearchDirectory != null)
            {
                GetFileArgsViaSearch(commandArgs);
            }

            commandArgs.Append(base.ToString());

            return commandArgs.ToString();
        }

        private void GetFileArgsViaSearch(StringBuilder commandArgs)
        {
            if (PatternSearchOption == SearchOption.AllDirectories)
            {
                if (TraverseSubrepositories)
                    commandArgs.Append(" -S");

                var inPats = IncludePatterns as List<String>;
                if (inPats != null && inPats.Count > 0)
                {
                    commandArgs.Append(" -I");
                    foreach (var inPat in inPats)
                    {
                        commandArgs.Append(String.Concat(" ", inPat));
                    }
                }

                var exPats = ExcludePatterns as List<String>;
                if (exPats != null && exPats.Count > 0)
                {
                    commandArgs.Append(" -X");
                    foreach (var exPat in exPats)
                    {
                        commandArgs.Append(String.Concat(" ", exPat));
                    }
                }
            }

            if (PatternSearchOption == SearchOption.TopDirectoryOnly)
            {
                var inPats = IncludePatterns as List<String>;
                var includeFiles = new List<String>();

                if (inPats != null && inPats.Count > 0)
                {
                    // Build a list of file paths that match each input pattern.
                    foreach (var inPat in inPats)
                    {
                        includeFiles.AddRange(Directory.EnumerateFiles(SearchDirectory.FullName, inPat,
                                                                       PatternSearchOption));
                    }
                }
                else
                {
                    includeFiles.AddRange(Directory.EnumerateFiles(SearchDirectory.FullName, "*",
                                                                   SearchOption.TopDirectoryOnly));
                }

                // Now remove any that match the exclude patterns.
                var exPats = ExcludePatterns as List<String>;
                var excludeFiles = new List<String>();

                if (exPats != null && exPats.Count > 0)
                {
                    foreach (var exPat in exPats)
                    {
                        excludeFiles.AddRange(Directory.EnumerateFiles(SearchDirectory.FullName, exPat,
                                                                       PatternSearchOption));
                    }
                }

                List<String> files;
                if (excludeFiles.Count > 0)
                    files = includeFiles.SkipWhile(x => !excludeFiles.Contains(x)) as List<String>;
                else
                    files = includeFiles;

                if (files != null && files.Count > 0)
                    commandArgs.Append(files.Aggregate(" ",(x, y) => String.Concat(x, " ", y)));
            }
        }

        #endregion
    }
}