﻿using System;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.IO;
using System.Text;

namespace PCG.Mercurial
{
    /// \<summary>
    /// My custom Mercurial command class.
    /// \</summary>
    ///
    /// \<remarks>
    /// Description of what command this class implements. Also highlight any
    /// dependencies, assumptions, gotchas, etc.
    /// \</remarks>
    ///
    /// \<example>
    /// Preferred: var command = CatCommand.CreateCommand(
    /// \<see cref="DirectoryInfo"/> repository, Object otherRequiredParams); 
    /// var command = new CatCommand();
    /// \</example>
    public class CatCommand : HgCommandBase
    {

        #region Constructors
     // ####################################################################
     // ###### Constructors
     // ####################################################################
        private CatCommand(DirectoryInfo repository) : base(repository)
        {
            Contract.Requires(repository != null);
        }

        #endregion


        #region Static Methods
     // ####################################################################
     // ###### Static Methods
     // ####################################################################
        /// \<summary>
        /// Default class factory method.
        /// \</summary>
        /// \<param name="repository"></param>
        /// \<returns></returns>
        public static CatCommand CreateCommand(DirectoryInfo repository)
        {
            return new CatCommand(repository);
        }
     
        #endregion


        #region Constants
     // ####################################################################
     // ###### Constants
     // ####################################################################
        private const String CommandText = "cat";

        #endregion


        #region Instance Fields
     // ####################################################################
     // ###### Instance Fields
     // ####################################################################

        #endregion


        #region Properties
     // ####################################################################
     // ###### Properties
     // ####################################################################
        public Boolean Decode { get; set; }
        public String OutputFormat { get; set; }
        public String IncludePattern { get; set; }
        public String ExcludePattern { get; set; }
        public String Revision { get; set; }
        #endregion


        #region Instance Methods
     // ####################################################################
     // ###### Instance Methods
     // ####################################################################

        #endregion

        
        #region Base Class Overrides
     // ####################################################################
     // ###### Base Class Overrides
     // ####################################################################

        /// \<summary>
        /// Returns a properly populated command result based on the results of the command's execution.
        /// \</summary>
        /// \<param name="hgProcess"></param>
        /// \<returns></returns>
        public override IHgCommandResult GetCommandResult(Process hgProcess)
        {
            Contract.Requires(hgProcess != null);

            return hgProcess.HasExited ? new CatCommandResult(hgProcess) : null;
        }
        /// \<summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:PCG.Mercurial.IHgCommand"/>.
        /// \</summary>
        /// \<returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:PCG.Mercurial.HgCommandBase"/>.
        /// \</returns>
        /// \<filterpriority>2</filterpriority>
        /// \<exception cref="NullReferenceException">Repository is required.</exception>
        public override string ToString()
        {
            Contract.Ensures(!String.IsNullOrEmpty(Contract.Result<String>()));

            // Build your hg.exe command line arguments.
            var commandArgs = new StringBuilder(CommandText); // e.g. (hg.exe) myaction ...

            #region Your command string options.
            if (Decode)
                commandArgs.Append(" --decode");

            if (!String.IsNullOrEmpty(OutputFormat))
                commandArgs.Append(String.Format(" --output {0}", OutputFormat));

            if (!String.IsNullOrEmpty(IncludePattern))
                commandArgs.Append(String.Format(" --include {0}", IncludePattern));

            if (!String.IsNullOrEmpty(ExcludePattern))
                commandArgs.Append(String.Format(" --exclude {0}", ExcludePattern));

            if (!String.IsNullOrEmpty(Revision))
                commandArgs.Append(String.Format(" --rev {0}", Revision));

            #endregion

            // Call down to base class to append hg's global options.
            commandArgs.Append(base.ToString()); // e.g. (hg.exe) myaction -v -R c:\temp\myrepo -y

            // Return the "serialized" command.
            return commandArgs.ToString();
        }

        #endregion

    }
}