﻿using System;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.IO;
using System.Text;

namespace PCG.Mercurial
{
    /// \<summary>
    /// My custom Mercurial command class.
    /// \</summary>
    ///
    /// \<remarks>
    /// Description of what command this class implements. Also highlight any
    /// dependencies, assumptions, gotchas, etc.
    /// \</remarks>
    ///
    /// \<example>
    /// Preferred: var command = CommitCommand.CreateCommand(
    /// \<see cref="DirectoryInfo"/> repository, Object otherRequiredParams); 
    /// var command = new CommitCommand();
    /// \</example>
    public class CommitCommand : HgCommandBase
    {

        #region Constructors
     // ####################################################################
     // ###### Constructors
     // ####################################################################
        private CommitCommand(DirectoryInfo repository) : base(repository)
        {
            Contract.Requires(repository != null);
        }

        #endregion


        #region Static Methods
     // ####################################################################
     // ###### Static Methods
     // ####################################################################
        /// \<summary>
        /// Default class factory method.
        /// \</summary>
        /// \<param name="repository"></param>
        /// <param name="message"></param>
        /// \<returns></returns>
        public static CommitCommand CreateCommand(DirectoryInfo repository, String message)
        {
            Contract.Requires(!String.IsNullOrEmpty(message));
            return new CommitCommand(repository) {Message = message};
        }

        #endregion


        #region Constants
     // ####################################################################
     // ###### Constants
     // ####################################################################
        private const string CommitMessageMissingException = "A commit message must be provided when running the command non-interactively.";
        private const String CommandText = "commit";

        #endregion


        #region Instance Fields
     // ####################################################################
     // ###### Instance Fields
     // ####################################################################

        #endregion


        #region Properties
     // ####################################################################
     // ###### Properties
     // ####################################################################
        /// <summary>
        /// Commit message to be used.
        /// </summary>
        /// <remarks>
        /// If this value is not set, and
        /// Interactive is set to <see langword="false"/>, then it will 
        /// throw an exception.
        /// </remarks>
        public String Message { get; set; }

        #endregion


        #region Instance Methods
     // ####################################################################
     // ###### Instance Methods
     // ####################################################################

        #endregion

        
        #region Base Class Overrides
     // ####################################################################
     // ###### Base Class Overrides
     // ####################################################################

        /// \<summary>
        /// Returns a properly populated command result based on the results of the command's execution.
        /// \</summary>
        /// \<param name="hgProcess"></param>
        /// \<returns></returns>
        public override IHgCommandResult GetCommandResult(Process hgProcess)
        {
            Contract.Requires(hgProcess != null);

            return hgProcess.HasExited ? new CommitCommandResult(hgProcess) : null;
        }
        /// \<summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:PCG.Mercurial.IHgCommand"/>.
        /// \</summary>
        /// \<returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:PCG.Mercurial.HgCommandBase"/>.
        /// \</returns>
        /// \<filterpriority>2</filterpriority>
        /// \<exception cref="NullReferenceException">Repository is required.</exception>
        /// <exception cref="InvalidOperationException">A commit message must be provided when running the command non-interactively.</exception>
        public override string ToString()
        {
            Contract.Ensures(!String.IsNullOrEmpty(Contract.Result<String>()));

            // Build your hg.exe command line arguments.
            var commandArgs = new StringBuilder(CommandText); // e.g. (hg.exe) myaction ...

            #region Build your command string options here.

            // Because this method is an override, we cannot use Contract.Requires.
            if (String.IsNullOrEmpty(Message) && Interactive == false)
                throw new InvalidOperationException(CommitMessageMissingException);
            
            if (!String.IsNullOrEmpty(Message))
                commandArgs.Append(String.Format(" -m \"{0}\"", Message));

            #endregion

            // Call down to base class to append hg's global options.
            commandArgs.Append(base.ToString()); // e.g. (hg.exe) myaction -v -R c:\temp\myrepo -y

            // Return the "serialized" command.
            return commandArgs.ToString();
        }

        #endregion

    }
}