﻿using System;
using System.Data;
using System.Diagnostics;
using System.Diagnostics.Contracts;
using System.IO;
using System.Linq;
using IniParser;
using PCG.Mercurial.Properties;

namespace PCG.Mercurial
{
	/// <summary>
	/// Provides a C# wrapper around the Mercurial (hg) executable.
	/// </summary>
	/// 
	/// <discussion>
	/// This class provides a series of convenience methods designed
	/// to facilitate common Mercurial use cases.  If you have a need
	/// that is not represented here, you can create your own HgCommand
	/// object and pass it to one of the service methods, like
	/// <see cref="RunSynchronousCommand"/>().
	/// </discussion>

	public class RepositoryManager : IRepositoryManager
	{        
		#region Static Methods

		/// <summary>
		/// Static factory method.  Returns null of the hg executable cannot be found.
		/// Will use the file path you provide, or will use the value provided in the path,
		/// or will search every mounted drive for the executable's name.
		/// </summary>
		/// <returns></returns>
		
		public static RepositoryManager Create(DirectoryInfo repositoryLocation, String filePathToExecutable = @"")
		{
			Contract.Requires(repositoryLocation != null && repositoryLocation.Exists);


			var hgPath = filePathToExecutable;

			if (String.IsNullOrEmpty(hgPath))
			{
				// Long way: Find the executable by searching all mounted drives.
				hgPath = SearchPathForHg();
			}

			if (String.IsNullOrEmpty(hgPath))
			{
				// Long way: Find the executable by searching all mounted drives.
				hgPath = SearchFileSystemForHgPath();
			}

			var hgLocation = new FileInfo(hgPath);

			return hgLocation.Exists ? new RepositoryManager { ExecutableInfo = hgLocation, RepositoryLocation = repositoryLocation } : null;
		}

		/// <summary>
		/// Search the PATH environment variable for the Mercurial executable.
		/// </summary>
		/// <returns></returns>
		private static String SearchPathForHg()
		{
			// Look for TortoiseHG in the PATH variable.
			var path = Environment.GetEnvironmentVariable("Path") ?? String.Empty;

			var pathDirectories = path.Split(";".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);

			return pathDirectories.Select(pathDirectory => String.Concat(pathDirectory, Settings.Default.ExecutableName)).FirstOrDefault(File.Exists);
		}

		/// <summary>
		/// Performs a laborious search of the filesystem for the Hg executable.
		/// </summary>
		/// <returns></returns>
		private static string SearchFileSystemForHgPath()
		{
			string hgPath = String.Empty;

			var drives = DriveInfo.GetDrives();

			foreach (var driveInfo in drives)
			{
				try
				{
					var results = Directory.GetFiles(driveInfo.RootDirectory.FullName, Settings.Default.ExecutableName, SearchOption.AllDirectories);
					if (results.Length == 1)
						hgPath = results[0];
				}
				catch (Exception exception)
				{
					Trace.TraceError("Error while searching drive {0} for '{1}': {2}", driveInfo.Name, Settings.Default.ExecutableName, exception);
				}
			}
			return hgPath;
		}

		#endregion

		private IniData _hgrc;

		#region Properties

		/// <summary>
		/// Returns the file path information for the Mercurial executable.
		/// </summary>
		public FileInfo ExecutableInfo { get; set; }

		/// <summary>
		/// Returns the name of the Mercurial executable defined in the application's settings file.
		/// </summary>
		public String DefaultExecutableName { get { return Settings.Default.ExecutableName; } }

		/// <summary>
		/// The location of the Mercurial repository.
		/// </summary>
		public DirectoryInfo RepositoryLocation { get; set; }

		/// <exception cref="FileNotFoundException"><c>FileNotFoundException</c>.</exception>
		public IniData HgConfiguration
		{
			get
			{
				if (_hgrc == null)
				{
					var rcFilePath = Path.Combine(RepositoryLocation.FullName, ".hg", "hgrc");
					var parser = new FileIniDataParser();
					if (!File.Exists(rcFilePath))
						throw new FileNotFoundException();
					_hgrc = parser.LoadFile(rcFilePath, true);
				}
				return _hgrc;
			}
			set
			{
				var rcFilePath = Path.Combine(RepositoryLocation.FullName, ".hg", "hgrc");
				if (!File.Exists(rcFilePath))
					throw new FileNotFoundException();
				var parser = new FileIniDataParser();
				_hgrc = value;
				parser.SaveFile(rcFilePath, _hgrc);
			}
		}

		#endregion


		#region Instance Methods

		/// <summary>
		/// Runs the Mercurial executable using the provided command object.
		/// If the command's interactive flag is set, the method will return
		/// with the process waiting for interactive input from stdin.
		/// Otherwise, the process will have exited already.  The buffers
		/// will not have been processed, however.
		/// </summary>
		/// <param name="command"></param>
		/// <returns></returns>
		/// <exception cref="NoNullAllowedException">The command arguments cannot be <see langword="null"/> or empty.</exception>
		public Process RunSynchronousCommand(HgCommandBase command)
		{
			Contract.Requires(command != null && command.Repository != null);

			var commandArgs = command.ToString();

			if (String.IsNullOrEmpty(commandArgs))
				throw new NoNullAllowedException("The command arguments cannot be null or empty.");

			var hgProcess = new Process
			{
				StartInfo =
				{
					WorkingDirectory = command.Repository.FullName,
					CreateNoWindow = true,
					WindowStyle = ProcessWindowStyle.Hidden,
					UseShellExecute = false,
					RedirectStandardOutput = true,
					RedirectStandardError = true,
					FileName = this.ExecutableInfo == null? Settings.Default.ExecutableName : this.ExecutableInfo.ToString(),
					Arguments = commandArgs
				},
				EnableRaisingEvents = false
			};

			if (command.Interactive)
			{
				hgProcess.StartInfo.RedirectStandardInput = true;
				hgProcess.Start();
				hgProcess.WaitForInputIdle();
			}
			else
			{
				hgProcess.Start();
				hgProcess.WaitForExit();
			}
			
			return hgProcess;
		}

		#endregion


		#region Convenience Methods
		// ####################################################################
		// ###### Action Methods                                         ######
		// ####################################################################

		/// <summary>
		/// Initializes a new repository at the directory provided.
		/// </summary>
		/// <returns></returns>
		/// <exception cref="NullReferenceException"><see cref="RunSynchronousCommand"/> requires a valid <see cref="InitCommand"/> object.</exception>
		public HgResult Init(DirectoryInfo repoInfo)
		{
			Contract.Requires(repoInfo != null);

			Trace.WriteLine("Preparing to call `hg init`", "Repository Manager");

			HgCommandBase command = new InitCommand(repoInfo);

			if (command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if (command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = command.GetCommandResult(process);

			// NOTE.ZJG: We could do some regex on the output, etc.,
			// but we're good to go here.

			return result != null ? result.ExitCode : HgResult.UnspecifiedError;
		}

		public HgResult Clone(string repositoryUrl, string username, string password)
		{
			//Contract.Requires(repoInfo != null);

			Trace.WriteLine("Preparing to call `hg clone`", "Repository Manager");

			HgCommandBase command = CloneCommand.CreateCommand(RepositoryLocation, repositoryUrl);

			if(command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if(command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = command.GetCommandResult(process);

			return result != null ? result.ExitCode : HgResult.UnspecifiedError;
		}

		public IncomingCommandResult Incoming()
		{
			//Contract.Requires(repoInfo != null);

			Trace.WriteLine("Preparing to call `hg incoming`", "Repository Manager");

			HgCommandBase command = IncomingCommand.CreateCommand(RepositoryLocation);

			if(command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if(command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = ((IncomingCommand)command).GetCommandResult(process);

			return result as IncomingCommandResult;
		}

		public HgResult Pull()
		{
			//Contract.Requires(repoInfo != null);

			Trace.WriteLine("Preparing to call `hg incoming`", "Repository Manager");

			HgCommandBase command = PullCommand.CreateCommand(RepositoryLocation);

			if(command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if(command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = command.GetCommandResult(process);

			return result != null ? result.ExitCode : HgResult.UnspecifiedError;
		}

		/// <summary>
		/// Commits a new changeset to the repository.
		/// </summary>
		/// <returns></returns>
		/// <exception cref="NullReferenceException"><see cref="RunSynchronousCommand"/> requires a valid <see cref="CommitCommand"/> object.</exception>
		public HgResult CommitAllChanges(DirectoryInfo repoInfo, String commitMessage)
		{
			Contract.Requires(repoInfo != null);

			Trace.WriteLine("Preparing to call `hg commit`", "Repository Manager");

			HgCommandBase command = CommitCommand.CreateCommand(repoInfo, commitMessage);

			if (command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid CommitCommand object.");

			if (command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid CommitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = command.GetCommandResult(process);

			// NOTE.ZJG: We could do some regex on the output, etc.,
			// but we're good to go here.
			if (result.ExitCode == HgResult.CommitFileSuccess)
				Trace.WriteLine(String.Format("({0}) Success!", result.ExitCode, result.ResultData), "Repository Manager");
			else
				Trace.WriteLine(String.Format("({0}) {1}", result.ExitCode, result.ErrorData), "Repository Manager");

				return result.ExitCode;
		}

		/// <summary>
		/// Adds (a) new file(s) to a repository.
		/// </summary>
		/// <returns></returns>
		/// <exception cref="NullReferenceException"><see cref="RunSynchronousCommand"/> requires a valid <see cref="InitCommand"/> object.</exception>
		public HgResult Add(DirectoryInfo repoInfo, FileInfo[] newFiles)
		{
			Contract.Requires(newFiles != null);
			Contract.Requires(repoInfo != null);

			Trace.WriteLine("Preparing to call `hg add`", "Repository Manager");

			HgCommandBase command = AddFileCommand.CreateCommand(repoInfo, newFiles);

			if (command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if (command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = command.GetCommandResult(process);

			// NOTE.ZJG: We could do some regex on the output, etc.,
			// but we're good to go here.

			return result != null ? result.ExitCode : HgResult.UnspecifiedError;
		}

		/// <summary>
		/// Adds all files in a subdirectory to a repository.
		/// </summary>
		/// <returns></returns>
		/// <exception cref="NullReferenceException"><see cref="RunSynchronousCommand"/> requires a valid <see cref="InitCommand"/> object.</exception>
		public HgResult Add(DirectoryInfo repoInfo, DirectoryInfo subDirectory, Boolean recursively)
		{
			Trace.WriteLine("Preparing to call `hg add`", "Repository Manager");

			HgCommandBase command = null;

			if (recursively)
				command = AddFileCommand.CreateCommand(repoInfo, subDirectory,SearchOption.AllDirectories, true);
			else
				command = AddFileCommand.CreateCommand(repoInfo, subDirectory, SearchOption.TopDirectoryOnly, false);

			if (command == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand object.");

			if (command.Repository == null)
				throw new NullReferenceException("RunSynchronousCommand requires a valid InitCommand.Repository object.");

			var process = RunSynchronousCommand(command);
			var result = command.GetCommandResult(process);

			// NOTE.ZJG: We could do some regex on the output, etc.,
			// but we're good to go here.

			return result.ExitCode;
		}

		#endregion
	}
}
